package com.bull.cimero.pluginEditor.generator.serviceMixGenerator;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.generator
				.serviceMixGenerator.model.IModelServiceMix;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * abstract class of generators (jar and xml).
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class GenericGenerator {

	/** the model to use. */
	private Diagram diagram = null;
	/** the file to use. */
	private IFile file = null;

	/**
	 * copy constructor.
	 * @param diagramParam : diagram to use
	 * @param fileParam : IFile associated to the model
	 */
	public GenericGenerator(final Diagram diagramParam
			, final IFile fileParam) {
		setDiagram(diagramParam);
		setFile(fileParam);
	}

	/**
	 * abstract method which will be surcharged
	 * in jarGenerator and xmlGenerator.
	 * @return true if the generation is ok
	 * @throws Exception
	 * 			throw exception if the generation model of ServiceMix
	 * 			is impossible
	 * @throws
	 */
	public abstract boolean generateConfiguration() throws Exception;

	/**
	 * abstract method which will be surcharged
	 * in subClass.
	 * @return the serviceMix model
	 */
	protected abstract IModelServiceMix generateModel();

	/**
	 * abstract method which will be surcharged
	 * in subClass.
	 * @param conLst : the list that contains all information to generate.
	 * @throws CoreException
	 * 			throw if JETEmitter cannot arrive to generate code
	 * @throws InvocationTargetException
	 * 			throw if the Jet specify file not exists
	 * @throws InterruptedException
	 * 			throw if the user stop the generation
	 * @throws
	 */
	protected final void generate(final List<ContainerEmitArgFile> conLst)
		throws CoreException, InvocationTargetException, InterruptedException {
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			public void execute(final IProgressMonitor monitor)
			throws CoreException, InvocationTargetException
			, InterruptedException {
				try {

					for (Iterator<ContainerEmitArgFile> it = conLst.iterator();
					it.hasNext();) {
						ContainerEmitArgFile myCon = it.next();
						
						if (myCon instanceof StaticCopyFile) {
							saveFile(((StaticCopyFile)myCon).getAbsoluteFilePath(), myCon.getNewFile(), monitor);
						} if (myCon instanceof StaticCopyFile) {
							
						}else{
							String generated = null;
							generated = myCon.getEmitter().generate(
									monitor, myCon.getArguments());
							if (generated.compareTo("") == 0) {
								//exception problem
								throw new InterruptedException(
										"error in JET generation file. "
										+ "Please see the file"
										+ " in .JETEmitter "
										+ "project");
							}
							saveGenerated(generated, myCon.getNewFile(), monitor);
						}
					}
				} catch (IOException ioe) {
					throw new InvocationTargetException(ioe);
				} catch (JETException e) {
					throw new CoreException(CimeroLog.newStatus(IStatus.ERROR
							, "exception in generation function"
							, e));
				}
			}
		};

		Shell shell = new Shell();
		ProgressMonitorDialog dialog = new ProgressMonitorDialog(shell);
		dialog.run(true, true, op);
	}

	/**
	 * abstract method which will be surcharged
	 * in subClass.
	 * @param generated : the generate code. It is a String
	 * @param newFile : the file where this code will be save
	 * @param monitor : the monitor dialog
	 * @throws IOException : throw if there
	 * 			is a problem during the file creation
	 * @throws CoreException : throw if the monitor crashs
	 */
	private void saveGenerated(final String generated,
			final IFile newFile, final IProgressMonitor monitor)
			throws IOException, CoreException {
		InputStream contents = new ByteArrayInputStream(generated.getBytes());

		IContainer parent = newFile.getParent();
		//IFile target = parent.getFile(new Path(newFile.getName() + ".xml"));
		IFile target = newFile;
		if (target.exists()) {
			target.setContents(contents, true, false, monitor);
		} else {
			File systemFile = target.getLocation().toFile();
			if (systemFile.exists()) {
				// check if out of sync
				parent.refreshLocal(1, monitor);
				// not user-friendly: user did not
				// request Refresh...
				target.setContents(contents, true, false, monitor);
			} else {
				target.create(contents, false, monitor);
			}
		}
	}

	private void saveFile(final String absoluteFilePath,
			final IFile newFile, final IProgressMonitor monitor)
			throws IOException, CoreException {
		
		
		String path = absoluteFilePath;	
		if (absoluteFilePath.contains("file:")) {
				path = absoluteFilePath.substring("file:".length());
		}
		
		InputStream contents =  new FileInputStream(path);
	
		IContainer parent = newFile.getParent();
		//IFile target = parent.getFile(new Path(newFile.getName() + ".xml"));
		IFile target = newFile;
		
		File systemFile = target.getLocation().toFile();
		if (systemFile.exists()) {
				// check if out of sync
				parent.refreshLocal(1, monitor);
				// not user-friendly: user did not
				// request Refresh...
				target.setContents(contents, true, false, monitor);
		} else {
				target.create(contents, false, monitor);
		}
	
	}
	
	/**
	 * get the diagram.
	 * @return Returns the diagram.
	 */
	public final Diagram getDiagram() {
		return diagram;
	}

	/**
	 * set the given diagram.
	 * @param diagramParam The diagram to set.
	 */
	public final void setDiagram(final Diagram diagramParam) {
		this.diagram = diagramParam;
	}

	/**
	 * get the file.
	 * @return Returns the file.
	 */
	public final IFile getFile() {
		return file;
	}

	/**
	 * set the given file.
	 * @param fileParam The file to set.
	 */
	public final void setFile(final IFile fileParam) {
		this.file = fileParam;
	}

	/**
	 * This inner class contains all information to generate
	 * configuration with JET.
	 * @author Cedric Mouilleron
	 *
	 */
	protected class ContainerEmitArgFile {
		/**.*/
		protected  JETEmitter emitter = null;
		/**.*/
		protected Object[] arguments = null;
		/**.*/
		protected IFile newFile = null;

		/**
		 * default constructor.
		 * @param em : the emmiter.
		 * @param obj : the object table.
		 * @param fileP : the IFile .
		 */
		public ContainerEmitArgFile(){
			
		}
		public ContainerEmitArgFile(final JETEmitter em,
				final Object[] obj, final IFile fileP) {
			this.emitter = em;
			this.arguments = obj;
			this.newFile = fileP;
		}

		/**
		 * @return the arguments
		 */
		public final Object[] getArguments() {
			return arguments;
		}

		/**
		 * @return the emitter
		 */
		public final JETEmitter getEmitter() {
			return emitter;
		}

		/**
		 * @return the newFile
		 */
		public final IFile getNewFile() {
			return newFile;
		}
	}
	
	/**
	 * This inner class contains all information to generate
	 * configuration with JET.
	 * @author Cedric Mouilleron
	 *
	 */
	protected class StaticCopyFile extends ContainerEmitArgFile {
		
		private String absoluteFilePath = null;
		
		
		public StaticCopyFile(String absolutePath,  final IFile fileP) {
			this.absoluteFilePath = absolutePath;
			this.newFile = fileP;
		}


		public String getAbsoluteFilePath() {
			return absoluteFilePath;
		}


		public void setAbsoluteFilePath(String absoluteFilePath) {
			this.absoluteFilePath = absoluteFilePath;
		}


		

		
		
	}
	
protected class ProjectInternalFile extends ContainerEmitArgFile {
		
		private IFile srcFile = null;
		
		
		public ProjectInternalFile(IFile srcFile,  final IFile fileP) {
			this.srcFile = srcFile;
			this.newFile = fileP;
		}


		public IFile getSrcFile() {
			return srcFile;
		}


		public void setSrcFile(IFile srcFile) {
			this.srcFile = srcFile;
		}


			
	}
}
