package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.RSS;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ConstructorClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.PropertyClass;

/**
 * This class is the rss factory. It is used to generate the servicemix model
 * that represents a rss component.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class RSSFactory extends BCLWFactory {

	/**XPATH class expression.*/
	private static final String XPATHEXPR =
		"org.apache.servicemix.expression.JaxenStringXPathExpression";
	/**Constante expression.*/
	private static final String CONSTANTEEXPR =
		"org.apache.servicemix.expression.ConstantExpression";
	/**the class using by ServiceMix to the Email poller.*/
	private static final String CLASSIN =
		"org.apache.servicemix.components.rss.RssPollingComponent";
	/**The class using by the rss sender into servicemix.*/
	private static final String CLASSOUT =
		"org.apache.servicemix.components.rss.FeedWriter";
	/**the output type using in rss poller.*/
	private static final String OUTPUTTYPE = "outputType";
	/**The period using by the rss poller.*/
	private static final String PERIOD = "period";
	/**the last polled date property.*/
	private static final String LASTPOLLEDDATE = "lastPolledDate";
	/**The urlString property.*/
	private static final String URLSSTRING = "urlStrings";
	/**The feedfile property name.*/
	private static final String FEEDFILE = "feedFile";
	/**The entry title property name.*/
	private static final String ENTRYTITLE = "entryTitle";
	/**The entry value property name.*/
	private static final String ENTRYVALUE = "entryValue";
	/**The maxi entry count.*/
	private static final String MAXENTRY = "maximumEntryCount";


	//CONSTRUCTOR
	/**constructor with the mep mode in parameter.
	 * @param mepMode : the mep mode of this component. True if INMEP
	 * @param ref : the reference of the current namespace.
	 * */
	public RSSFactory(final NameSpaceClass ref, final Boolean mepMode) {
		super(ref, mepMode);
	}

	/**
	 * This function create the rss servicemix model.
	 * @param elt : the element model that will be used to generate
	 * 		this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec created
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof RSS)) {
			if ((elt.isValide(false)) || (force)) {
				RSS monRss = (RSS) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				//set the name
				myActi.setComponentName(monRss.getName());
				//set the service
				myActi.setServiceName(monRss.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//it is an INMEP => RSS POLLER
					//set the destination service
					setDestinationService(myActi , monRss);
				}
				//create the components
				ComponentClass compo = createComponent(monRss);
				myActi.setComponent(compo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");
	}

	/**
	 * This function creates the component of the servicemix rss element.
	 * @param monRSS : the cimero model element
	 * @return the instance of the component class.
	 */
	private ComponentClass createComponent(final RSS monRSS) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = null;
		if (monRSS.isInput()) {
			monB = createBeanRSSOUTMEP(monRSS);
		} else {
			monB = createBeanRSSINMEP(monRSS);
		}
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates the bean using by the rss in INMEP in the
	 * serviceMix model.
	 * @param monRss : the rss cimero model.
	 * @return the bean routeur.
	 */
	private LWBeanClass createBeanRSSINMEP(final RSS monRss) {
		//create the bean
		LWBeanClass beanRss = new LWBeanClass(CLASSIN);
		//add the property rss type to this bean
		beanRss.addProperty(
				createProperty(OUTPUTTYPE, monRss.getFlowType()) , "");
		//add the period of the poller
		beanRss.addProperty(
				createProperty(PERIOD ,
						String.valueOf(monRss.getReadFrequence())) , "");
		//add the urls of the rss files
		//
		String [] tabStr = monRss.getUrls().split(";");
		beanRss.addProperty(
				createProperty(URLSSTRING , tabStr) , "");
		//add the last polled date.
		String polledDate = monRss.getLastPollDate();
		//create the lastpolleddate property
		PropertyClass lastPolledDateProp = new PropertyClass(LASTPOLLEDDATE);
		//create the bean constructor
		LWBeanClass lastPolledBean = new LWBeanClass("java.util.Date");
		lastPolledBean.setConstructor(new ConstructorClass(polledDate));
		lastPolledDateProp.setBean(lastPolledBean);
		beanRss.addProperty(lastPolledDateProp, "");
		return beanRss;
	}

	/**
	 * This function creates the bean using by the mail in OUTMEP in the
	 * serviceMix model.
	 * @param monRss : the cimero element
	 * @return the bean routeur.
	 */
	private LWBeanClass createBeanRSSOUTMEP(final RSS monRss) {
		LWBeanClass beanRSS = new LWBeanClass(CLASSOUT);
		//add the feedfile property
		beanRSS.addProperty(createProperty(FEEDFILE , monRss.getRssPath())
				, "");
		beanRSS.addProperty(createBeanConstructor(ENTRYTITLE , CONSTANTEEXPR ,
				monRss.getTitle()), "");
		if (monRss.getBody().compareTo("") != 0) {
			String regle = "concat(' '," + monRss.getBody() + ")";
			beanRSS.addProperty(createBeanConstructor(ENTRYVALUE, XPATHEXPR,
					regle), "");
		}
		beanRSS.addProperty(createProperty(MAXENTRY , "10"), "");
		return beanRSS;
	}

}
