package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.xmlGenerator;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.GenericGenerator;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.IModelServiceMix;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.DroolsRouterModel;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.DroolsRouterModel.ServiceMixRule;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * class used to generate an XML configuration file
 * from a cimero file.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class XmlGenerator extends GenericGenerator {

	/** constant for yes button in the msgbox. */
	public static final int YESBUTTON = 64;
	/**To determine if the generation must to be force.*/
	private boolean isForce = false;

	/**
	 * copy constructor.
	 * @param diagramParam : diagram to use
	 * @param fileParam : IFile associated to the model
	 */
	public XmlGenerator(final Diagram diagramParam
			, final IFile fileParam) {
		super(diagramParam, fileParam);
	}

	/**
	 * method used to generate the XML configuration file.
	 * @see com.bull.cimero.pluginEditor.generator
	 *   .serviceMixGenerator.model.GenericGenerator#generateConfiguration()
	 * @return boolean
	 * @throws Exception
	 * 		throw this exception is there is a problem during the
	 * 		model generation
	 * throws IOException : if user doesn't want to replace existing file
	 */
	@Override
	public final boolean generateConfiguration() throws Exception {
		//test if the file already exists
		if (testFileAlreadyExist()) {
			//already exists
			//--> tell user to replace or not the file
			Shell monShell = new Shell();
			MessageBox msg = new MessageBox(monShell, SWT.YES | SWT.NO);
			msg.setMessage("The XML file already exists !"
					+ " Do you want to replace it ?");
			msg.setText("XML file already exists");
			//the messagebox is open
			if (msg.open() != YESBUTTON) {
				//if the user had pressed NO
				throw new IOException();
			}
		}
		//we generate the model
		IModelServiceMix monModel = generateModel();
		if (monModel == null) {
			throw new Exception("ServiceMix generation model problem");
		}
		//we call the templates and generate the result
//		String templatePath = CimeroEditorPlugin.getTemplatePath();
//		String jetFile = templatePath + "LWGeneration/Container.jet";
		String base = Platform.getPlugin(CimeroEditorPlugin.getPluginId())
		.getDescriptor().getInstallURL().toString();
		String jetFile = base + "templates/LWGeneration/Container.jet";
		ClassLoader classloader = getClass().getClassLoader();
		//generate the xml file.
		JETEmitter emitter = new JETEmitter(jetFile, classloader);
		emitter.addVariable("CIMEROEDITOR" ,
				CimeroEditorPlugin.getPluginId());
		//to the drools file
		DroolsRouterModel drools = generateDroolsModel();
		JETEmitter  emitterDrools = null;
		if (drools != null) {
			//String jetFileDrools =
			//	templatePath + "DroolsGeneration/Drools.jet";
			String jetFileDrools =
				base + "templates/DroolsGeneration/Drools.jet";
			emitterDrools = new JETEmitter(jetFileDrools, classloader);
			emitterDrools.addVariable("CIMEROEDITOR" ,
					CimeroEditorPlugin.getPluginId());
		}
		//execute the generation
		try {
			//XML configuration file generation
			IContainer parent = getFile().getParent();
			//transform the name of the file to create jar path
			//the jar file has the same path
			//and the same name than the cimero file
			//so we just have to delete the extension and to replace it
			String[] tabRegex = getFile().getName().split("\\.");
			String xmlPath = "";
			for (int i = 0; i < tabRegex.length - 1; i++) {
				xmlPath += tabRegex[i];
			}
			//we concat the extension (.jar)
			xmlPath += ".xml";
			//
			IFile target = parent.getFile(new Path(xmlPath));



			//create the list to the generation
			List < ContainerEmitArgFile > conLst =
				new ArrayList < ContainerEmitArgFile >();
			//add the main file to generate in the list
			conLst.add(new ContainerEmitArgFile(emitter,
					new Object [] {monModel}, target));
			//add the xml file in the list
			if (drools != null) {
				for (int i = 0; i < drools.lstRuleSize(); i++) {
					ServiceMixRule tmpRule = drools.getRuleElt(i);
					parent = getFile().getParent();
					//set the target name
					IFile targetDrools = parent.getFile(new Path(
							drools.getPrefixeName() + "droolsFile_"
							+ tmpRule.getName() + ".xml"));
					conLst.add(new ContainerEmitArgFile(emitterDrools,
							new Object [] {tmpRule}, targetDrools));
				}
			}
			//generate the cml configuration
			generate(conLst);
		} catch (Exception e) {
			CimeroLog.errorLog("error during generation" , e);
			return false;
		}
		return true;
	}

	/**
	 * This function generate the model of ServiceMix using to
	 * generate a Jar configuration file.
	 * @return the model generate
	 */
	public final IModelServiceMix generateModel() {
		Diagram cimeroDiagram = getDiagram();
		IModelServiceMix smModel  = null;
		if (cimeroDiagram != null) {
			if (cimeroDiagram.getLstCimeroObject().size() != 0) {
				//Create the jarmodelFactory
				XmlModelFactory facto = new XmlModelFactory(
						cimeroDiagram, isForce);
				smModel = facto.generateModelServiceMix();
			} else {
				//error
				return null;
			}
		} else {
			//error
			return null;
		}
		return smModel;
	}

	/**
	 * This function is used to create the object need to create
	 * the drools files.
	 * @return the DroolsRouterModel if there is router in the
	 * 	cimero model or null in the other case.
	 */
	private DroolsRouterModel generateDroolsModel() {
		//we test if there is router in cimeroModel
		List < Routeur > lst = new ArrayList < Routeur >();
		Diagram diagTmp = getDiagram();
		if (diagTmp != null) {
			for (int i = 0; i < diagTmp.getLstCimeroObject().size(); i++) {
				GeneriqueCimeroModel elt = (GeneriqueCimeroModel) diagTmp
				.getLstCimeroObject().get(i);
				if (elt instanceof Routeur) {
					lst.add((Routeur) elt);
				}
			}
			//we test if there is at least one routeur in the list
			if (lst.size() != 0) {
				DroolsRouterModel droolsModel = new DroolsRouterModel();
				droolsModel.setPrefixeName(diagTmp.getCimeroDiagramName());
				for (int i = 0; i < lst.size(); i++) {
					Routeur routerLstElt = lst.get(i);
					droolsModel.addRule(routerLstElt.getName(),
							routerLstElt.getMaLstRule());
				}
				return droolsModel;
			} else {
				//there is no routeur in model
				return null;
			}
		} else {
			return null;
		}
	}

	/**
	 * method used to test if the wanted file already exists.
	 * @return boolean : true if already exist
	 */
	private boolean testFileAlreadyExist() {
		boolean bExist = false;
		IContainer parent = getFile().getParent();
		//transform the name of the file to create jar path
		//the jar file has the same path and the same name than the cimero file
		//so we just have to delete the extension and to replace it
		String[] tabRegex = getFile().getName().split("\\.");
		String xmlPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			xmlPath += tabRegex[i];
		}
		//we concat the extension (.jar)
		xmlPath += ".xml";
		//
		IFile target = parent.getFile(new Path(xmlPath));
		if (target.exists()) {
			bExist = true;
		} else {
			File systemFile = target.getLocation().toFile();
			if (systemFile.exists()) {
				bExist = true;
			}
		}
		return bExist;
	}

	/**
	 * set the force bolean to generate within validation.
	 * @param isForceParam the isForce to set
	 */
	public final void setForce(final boolean isForceParam) {
		this.isForce = isForceParam;
	}


}
