package com.bull.cimero.pluginEditor.utils.zoom;

import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.Viewport;
import org.eclipse.gef.editparts.ZoomListener;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.jface.util.ListenerList;

/**
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * class used to manage the zoom capacities of a digram.
 @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class DelegatingZoomManager extends ZoomManager implements ZoomListener {

	/** the current ZoomManager all work is delegated to. */
    private ZoomManager currentZoomManager;
    /** listeners. */
    private ListenerList zoomListeners = new ListenerList(NUMBEROFLISTENER);
    /** number of listener. */
	public static final int NUMBEROFLISTENER = 3;
    /** default zoom. */
	public static final double DEFAULTZOOM = 0.75;
    /** zoom 1. */
	public static final double ZOOM1 = 0.25;
    /** zoom 2. */
	public static final double ZOOM2 = 0.50;
    /** zoom 3. */
	public static final double ZOOM3 = 1;
    /** zoom 4. */
	public static final double ZOOM4 = 1.25;
    /** zoom 5. */
	public static final double ZOOM5 = 1.50;
    /** current zoom. */
	private double currentZoom = DEFAULTZOOM;

    /**
     * Creates a new DelegatingZoomManager instance.
     */
    public DelegatingZoomManager() {
        super((ScalableFigure) null, (Viewport) null);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomListener#zoomChanged(double)
     * @param zoom : new zoom
     */
    public final void zoomChanged(final double zoom) {
        Object[] listeners = zoomListeners.getListeners();
        for (int i = 0; i < listeners.length; ++i) {
            ((ZoomListener) listeners[i]).zoomChanged(zoom);
        }
        currentZoom = getZoom();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager
     * #addZoomListener(org.eclipse.gef.editparts.ZoomListener)
     * @param listener : listener to add
     */
    public final void addZoomListener(final ZoomListener listener) {
        zoomListeners.add(listener);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager
     * #removeZoomListener(org.eclipse.gef.editparts.ZoomListener)
     * @param listener : listener to remove
     */
    public final void removeZoomListener(final ZoomListener listener) {
        zoomListeners.remove(listener);
    }

    /**
     * Sets the ZoomManager all work should be delegated to.
     * @param zoomManager : zoom to set
     */
    public final void setCurrentZoomManager(final ZoomManager zoomManager) {
        if (null != currentZoomManager) {
            currentZoomManager.removeZoomListener(this);
        }

        currentZoomManager = zoomManager;
        setZoom(currentZoom);
        if (null != currentZoomManager) {
            currentZoomManager.addZoomListener(this);
            zoomChanged(currentZoomManager.getZoom());
        }
        double[] tab = {ZOOM1, ZOOM2, DEFAULTZOOM, ZOOM3, ZOOM4, ZOOM5};
        setZoomLevels(tab);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#canZoomIn()
     * @return boolean : true if possible to zoom in
     */
    public final boolean canZoomIn() {
        if (null == currentZoomManager) {
            return false;
        }
        return currentZoomManager.canZoomIn();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#canZoomOut()
     * @return boolean : true if possible to zoom out
     */
    public final boolean canZoomOut() {
        if (null == currentZoomManager) {
            return false;
        }
        return currentZoomManager.canZoomOut();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getZoom()
     * @return double : current zoom
     */
    public final double getZoom() {
        if (null == currentZoomManager) {
            return 1;
        }
        return currentZoomManager.getZoom();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getZoomAsText()
     * @return String : zoom as string
     */
    public final String getZoomAsText() {
        if (null == currentZoomManager) {
            return " 100%";
        }
        return currentZoomManager.getZoomAsText();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager
     * #getZoomLevelsAsText()
     * @return String[] : array of zoom as string
     */
    public final String[] getZoomLevelsAsText() {
        if (null == currentZoomManager) {
            return new String[] {" 100%"};
        }
        return currentZoomManager.getZoomLevelsAsText();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setZoom(double)
     * @param zoom : zoom to set
     */
    public final void setZoom(final double zoom) {
        if (null == currentZoomManager) {
            return;
        }
        currentZoomManager.setZoom(zoom);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager
     * #setZoomAsText(java.lang.String)
     * @param zoomString : zoom as string to set
     */
    public final void setZoomAsText(final String zoomString) {
        if (null == currentZoomManager) {
            return;
        }
        currentZoomManager.setZoomAsText(zoomString);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setZoomLevels(double[])
     * @param zoomLevels : zoom levels to set
     */
    public final void setZoomLevels(final double[] zoomLevels) {
        if (null == currentZoomManager) {
            return;
        }
        currentZoomManager.setZoomLevels(zoomLevels);
    }
}
