package com.bull.cimero.pluginEditor.validator;

import java.util.ArrayList;
import java.util.List;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * It is the implementation of he IValidator interface. This class
 * is used to validate a cimero file.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class Validator {

	/** activate or desactivate log information. */
	private boolean activateLog = true;

	/**
	 * This function is used to activate or desactivate log
	 * information.
	 * @param activateLogParam : true if messages logs must to be log
	 */
	public final void setActivateLog(final boolean activateLogParam) {
		this.activateLog = activateLogParam;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.validator.IValidator.validateFiles
	 * @param diagram : diagram to validate
	 * @param name : filename to the cimero file
	 * @return boolean
	 */
	public final boolean validate(final Diagram diagram, final String name) {
		if (diagram != null) {
			if (diagram.getLstCimeroObject().size() != 0) {
				//START THE VALIDATION
				startValidation(name);

				boolean returnValue;
				returnValue = true;
				//we verify that all the properties of all the element are
				//set.
				if (!verifyProperties(diagram)) {
					returnValue = false;
				} else {
					//if the property validation is clear,
					//we verify the diagram connections.
					if (!verifyConnection(diagram)) {
						returnValue = false;
					}
				}
				//STOP VALIDATION
				stopValidation(name);
				//
				return returnValue;
			} else {
				CimeroLog.errorLog("The validator raises an exception !"
						+ " The file in parameter is empty",
						new Exception());
				return false;
			}
		} else {
			CimeroLog.errorLog("The validator raises an exception !"
					+ " The file in parameter is not correct",
					new Exception());
			return false;
		}
	}

	/**
	 * This function is used to verify that all the connections
	 * of the elements which composing the cimeroDiagram?
	 * This function verifies that there is an input and output BC for all SE.
	 * @param cimeroDiagram : the diagram to analyse
	 * @return boolean
	 */
	private boolean verifyConnection(final Diagram cimeroDiagram) {
		boolean returnValue = true;
		for (int i = 0; i < cimeroDiagram.getLstCimeroObject().size(); i++) {
			GeneriqueCimeroModel tmp = (GeneriqueCimeroModel) cimeroDiagram
			.getLstCimeroObject().get(i);
			//is tmp is a SE
			if (tmp instanceof ServiceEngineModel) {
				//we research the BC in input
				if (!researchInput(null, tmp)) {
					/* BEGIN(AZ)
					sendLogMessage("SE name : " + tmp.getName() + "\nThis"
							+ " element hasn't reference directly "
							+ "or indirectly a "
							+ "binding component in input", 1);
					
					returnValue = false;
						END (AZ)*/
				} else {
					if (!researchOutput(null, tmp)) {
						/* BEGIN(AZ)
						sendLogMessage("SE name : " + tmp.getName()
								+ "\nThis element hasn't reference directly or "
								+ "indirectly a binding component in "
								+ "output", 1);
					
						returnValue = false;
								END (AZ)*/
					}
				}
			} else {
				//this is an BC
				//we verify that if this is an OUTPUT MEP, it have
				//an input connection
				BindingComponentModel tmpBC = (BindingComponentModel) tmp;
				if (tmpBC.isInput()) {
					if (!researchInput(null, tmp)) {
						/* BEGIN(AZ)
						sendLogMessage("BC name : " + tmp.getName() + "\nThis"
								+ " element hasn't reference directly or "
								+ "indirectly a "
								+ "binding component in input", 1);
						
						returnValue = false;
						END (AZ)*/
					}
				} else {
					if (!researchOutput(null, tmp)) {
						/* BEGIN(AZ)
						sendLogMessage("SE name : " + tmp.getName()
								+ "\nThis element hasn't reference directly or "
								+ "indirectly a binding component in "
								+ "output", 1);
						
						returnValue = false;
						END(AZ)*/
					}
				}
			}
		}
		return returnValue;
	}

	/**
	 * This function is used to verify that all the necessary properties
	 * of the elements which compose the cimeroDiagram are set?
	 * @param cimeroDiagram : the diagram to analyse
	 * @return boolean
	 */
	private boolean verifyProperties(final Diagram cimeroDiagram) {
		boolean returnResult = true;
		List cimeroElements = cimeroDiagram.getLstCimeroObject();
		for (int i = 0; i < cimeroElements.size(); i++) {
			GeneriqueCimeroModel tmp =
				(GeneriqueCimeroModel) cimeroElements.get(i);
			if (!tmp.isValide(true)) {
				returnResult = false;
			}
		}
		return returnResult;
	}

	/**
	 * This function add in log file that the validation begin.
	 * @param fileName : the file name using to the validation
	 */
	private void startValidation(final String fileName) {
		//we send a message in error log
		//sendLogMessage("starting the validation on "
		//		+ fileName + " file", 0);
	}

	/**
	 * This function add in log file that the validation is finish.
	 * @param fileName : the file name using to the validation.
	 */
	private void stopValidation(final String fileName) {
		//sendLogMessage("the validation on "
		//		+ fileName + " file is finished", 0);
	}

	/**
	 * This function is used to research if the element in param
	 * has a BC in input.
	 * This function return true if the element has an input connection.
	 * @param eltsAlreadySee : this list containts the elements
	 * that have already see in the function.
	 * @param elt : the element of reference
	 * @return boolean
	 */
	private boolean researchInput(
			List < GeneriqueCimeroModel > eltsAlreadySee,
			final GeneriqueCimeroModel elt) {
		//if eltsAlreadySee is null, we create it
		if (eltsAlreadySee == null) {
			eltsAlreadySee = new ArrayList < GeneriqueCimeroModel >();
		}
		//we verify if one of the incoming elements of elt is a BC
		List lstIncoming = elt.getIncomingConnections();
		for (int i = 0; i < lstIncoming.size(); i++) {
			GeneriqueCimeroModel tmp =
				((Connection) lstIncoming.get(i)).getSource();
			if (tmp instanceof BindingComponentModel) {
				return true;
			}
		}
		//we add in the list this elt
		eltsAlreadySee.add(elt);
		//elt did not have a BC directly in input. We call this
		//function for all the incoming msg.
		for (int i = 0; i < lstIncoming.size(); i++) {
			GeneriqueCimeroModel tmp =
				((Connection) lstIncoming.get(i)).getSource();
			//we verify that this element hasn't already see
			boolean alreadySee = false;
			for (int cpt = 0; cpt < eltsAlreadySee.size(); cpt++) {
				if (eltsAlreadySee.get(i) == tmp) {
					alreadySee = true;
				}
			}
			if ((!alreadySee) && (researchInput(eltsAlreadySee , tmp))) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This function is used to research if the element in param
	 * has a BC in output.
	 * This function return true if the element has an output connection.
	 * @param eltsAlreadySee : this list containts the elements
	 * that have already see in the function.
	 * @param elt : the element of reference
	 * @return boolean
	 */
	private boolean researchOutput(
			List < GeneriqueCimeroModel > eltsAlreadySee,
			final GeneriqueCimeroModel elt) {
		//if eltsAlreadySee is null, we create it
		if (eltsAlreadySee == null) {
			eltsAlreadySee = new ArrayList < GeneriqueCimeroModel >();
		}
		//we verify if one of the incoming elements of elt is a BC
		List lstOutcoming = elt.getOutgoingConnections();
		for (int i = 0; i < lstOutcoming.size(); i++) {
			GeneriqueCimeroModel tmp =
				((Connection) lstOutcoming.get(i)).getTarget();
			if (tmp instanceof BindingComponentModel) {
				return true;
			}
		}
		//we add in the list this elt
		eltsAlreadySee.add(elt);
		//elt did not have a BC directly in input. We call this
		//function for all the incoming msg.
		for (int i = 0; i < lstOutcoming.size(); i++) {
			GeneriqueCimeroModel tmp =
				((Connection) lstOutcoming.get(i)).getTarget();
			//we verify that this element hasn't already see
			boolean alreadySee = false;
			for (int cpt = 0; cpt < eltsAlreadySee.size(); cpt++) {
				if (eltsAlreadySee.get(cpt) == tmp) {
					alreadySee = true;
				}
			}
			if ((!alreadySee) && (researchOutput(eltsAlreadySee , tmp))) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This function is used to send message log to Eclipse.
	 * @param msg : the message
	 * @param type : the type of the log message
	 */
	private void sendLogMessage(final String msg, final int type) {
		if (activateLog) {
			if (type == 0) {
				//send a information log
				CimeroLog.infoLog(msg);
			} else if (type == 1) {
				//send an warning log
				CimeroLog.warningLog(msg);
			}
		}
	}
}
