package com.bull.cimero.pluginEditor.wizards;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.FTP;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JMS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Mail;
import com.bull.cimero.pluginEditor.editors.model.BCModel.RSS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ScreenOutput;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ServiceWeb;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Timer;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This is a sample new wizard. Its role is to create a new file
 * resource in the provided container. If the container resource
 * (a folder or a project) is selected in the workspace
 * when the wizard is opened, it will accept it as the target
 * container. The wizard creates one file with the extension
 * "cimero". If a sample multi-page editor (also available
 * as a template) is registered for the same extension, it will
 * be able to open it.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class CimeroNewWizard extends Wizard implements INewWizard {

	/** wizard page. */
	private CimeroNewWizardPage page;
	/** current selection of wizard. */
	private ISelection selection;

	/**
	 * Default constructor for CimeroNewWizard.
	 */
	public CimeroNewWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	/**
	 * Adding the page to the wizard.
	 */
	public final void addPages() {
		page = new CimeroNewWizardPage(selection);
		addPage(page);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 * @return boolean
	 */
	public final boolean performFinish() {
		final String containerName = page.getContainerName();
		final String fileName = page.getFileName();
		//we test if the file exists
		if (fileAlreadyExist(fileName, containerName)) {
			//we ask to the user if he want to replace the file
			MessageDialog msg = new MessageDialog(getShell(), "File exists",
					null, "This file already exists, "
					+ "do you want to replace it ?",
					MessageDialog.QUESTION, new String[] {"Yes", "No"}, 0);
			//the messagebox is open
			if (msg.open() == 1) {
				//return false and stop the file creation
				return false;
			} else {
				//try to close the editor (if it's open)
				DiagramEditor diag =
					CimeroRegistry.getInstance().getDiagramEditor(fileName);
				if (diag != null) {
					//only if the editor is open
					diag.closeEditor(false);
				}
			}
		}
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(final IProgressMonitor monitor)
			throws InvocationTargetException {
				try {
					doFinish(containerName, fileName, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(
					getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}

	/**
	 * This function test if a file with the filename exists in the projet
	 * specify by the containerName.
	 * @param fileName : the fileName to test
	 * @param containerName : the container where the file must to be test
	 * @return true if the file exists
	 */
	private boolean fileAlreadyExist(final String fileName,
			final String containerName) {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (!resource.exists() || !(resource instanceof IContainer)) {
			try {
				throwCoreException("Container \"" + containerName
						+ "\" does not exist.");
			} catch (CoreException e) {
				CimeroLog.errorLog("Error when tested if the file exists", e);
			}
		}
		IContainer container = (IContainer) resource;
		final IFile file = container.getFile(new Path(fileName));
		return file.exists();
	}

	/**
	 * The worker method. It will find the container, create the
	 * file if missing or just replace its contents, and open
	 * the editor on the newly created file.
	 * @param containerName : the container
	 * @param fileName : the current file
	 * @param monitor : current monitor
	 * @throws CoreException : if no ressource
	 */
	private void doFinish(final String containerName,
			final String fileName, final IProgressMonitor monitor)
	throws CoreException {
		// create a sample file
		monitor.beginTask("Creating " + fileName, 2);
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (!resource.exists() || !(resource instanceof IContainer)) {
			throwCoreException(
					"Container \"" + containerName + "\" does not exist.");
		}
		IContainer container = (IContainer) resource;
		final IFile file = container.getFile(new Path(fileName));
		try {
			InputStream stream = openContentStream(fileName);
			//tests if the file already exist
			if (file.exists()) {
				file.setContents(stream, true, true, monitor);
			} else {
				file.create(stream, true, monitor);
			}
			stream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		monitor.worked(1);
		monitor.setTaskName("Opening file for editing...");
		getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page2 = null;
				page2 = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page2, file, true);
				} catch (PartInitException e) {
					e.printStackTrace();
				}
			}
		});
		monitor.worked(1);
	}

	/**
	 * We will initialize file contents with a sample text.
	 * @param diagramName : the name of this diagram
	 * @return InputStream
	 */
	private InputStream openContentStream(final String diagramName) {
		Diagram cimeroDiagram = null;
		
		//no sample
		cimeroDiagram = new Diagram();
	
		//set the name diagram
		cimeroDiagram.setCimeroDiagramName(diagramName);
		InputStream istream = null;
		try {
			istream = cimeroDiagram.getAsStream();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return istream;
	}

	/**
	 * method to throw real exception.
	 * @param message : error message
	 * @throws CoreException : the wanted exception
	 */
	private void throwCoreException(final String message) throws CoreException {
		IStatus status = null;
		status = new Status(
				IStatus.ERROR, "CimeroEditor",
				IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 * @param workbench : the workbench
	 * @param selectionParam : current selection
	 */
	public final void init(
			final IWorkbench workbench,
			final IStructuredSelection selectionParam) {
		this.selection = selectionParam;
	}

	/**
	 * this function create an diagram with a complex sample
	 * in the new file.
	 * @return Diagram
	 */
	private Diagram createComplexSampleDiagram() {
		//
		Diagram diagram = new Diagram();
		//INPUT
		GeneriqueCimeroModel ftpModel = new FTP();
		GeneriqueCimeroModel rssModel = new RSS();
		GeneriqueCimeroModel mailModelIn = new Mail();
		//MIDDLE
		GeneriqueCimeroModel routeurFtp = new Routeur();
		GeneriqueCimeroModel routeurMail = new Routeur();
		GeneriqueCimeroModel routeurRss = new Routeur();
		//OUTPUT
		ServiceWeb serviceWebModelOut = new ServiceWeb();
		GeneriqueCimeroModel mailModelOut = new Mail();
		//LOCATION INPUT
		ftpModel.setLocation(new Point(100, 50));
		rssModel.setLocation(new Point(100, 250));
		mailModelIn.setLocation(new Point(100, 450));
		//LOCATION MIDDLE
		routeurFtp.setLocation(new Point(500, 50));
		routeurRss.setLocation(new Point(500, 250));
		routeurMail.setLocation(new Point(500, 450));
		//LOCATION OUTPUT
		serviceWebModelOut.setLocation(new Point(850, 150));
		mailModelOut.setLocation(new Point(850, 400));
		//MEP
		((BindingComponentModel) serviceWebModelOut).setInput(true);
		((BindingComponentModel) serviceWebModelOut).setOutput(false);
		((BindingComponentModel) mailModelOut).setInput(true);
		((BindingComponentModel) mailModelOut).setOutput(false);
		//CONNECTIONS
		new Connection(ftpModel, routeurFtp);
		new Connection(rssModel, routeurRss);
		new Connection(mailModelIn, routeurMail);
		//CONFIGURATION
		//MAIL IN
		mailModelIn.setName("My input mail");
		mailModelIn.setPropertyValue("login", "my_login");
		mailModelIn.setPropertyValue("password", "my_password");
		mailModelIn.setPropertyValue("inbox", "");
		mailModelIn.setPropertyValue("serveurName", "pop.free.fr");
		mailModelIn.setPropertyValue("portCommunication", "25");
		//RSS IN
		rssModel.setName("My input RSS");
		rssModel.setPropertyValue("polledfrequence", "5000");
		rssModel.setPropertyValue("urls"
				, "http://www.lefigaro.fr/rss/figaro_une.xml");
		//FTP IN
		ftpModel.setName("My input FTP");
		ftpModel.setPropertyValue("host adress", "127.0.0.1");
		ftpModel.setPropertyValue("login", "my_login");
		ftpModel.setPropertyValue("password", "my_password");
		ftpModel.setPropertyValue("path", "sharedFolder/");
		//MAIL OUT
		mailModelOut.setName("My output mail");
		mailModelOut.setPropertyValue("mailTo", "receiver@gmail.fr");
		mailModelOut.setPropertyValue("mailFrom", "sender@gmail.fr");
		mailModelOut.setPropertyValue("subject", "message subject");
		mailModelOut.setPropertyValue("mailBody", "message body");
		mailModelOut.setPropertyValue("attachement", "");
		mailModelOut.setPropertyValue("serveurName", "smtp.free.fr");
		mailModelOut.setPropertyValue("portCommunication", "25");
		//WEBSERVICE OUT
		serviceWebModelOut.setName("My output webService");
		serviceWebModelOut.setSoapRequest(true);
		serviceWebModelOut.setPropertyValue("serveurName"
				, "http://localhost:8080/axis/myJWS.jws?wsdl");
		//ROUTER MAIL 1
		routeurMail.setName("Mail router");
		((Routeur) routeurMail).setPropertyValue("number of comparaison", "2");
		//
		((Routeur) routeurMail).setPropertyValue(
				"1content / property", "1"); //property
		((Routeur) routeurMail).setPropertyValue(
				"1property name", "2");
		((Routeur) routeurMail).setPropertyValue(
				"1property value", "@hotmail.fr");
		((Routeur) routeurMail).setPropertyValue(
				"1comparaison type", "0"); //egal
		((Routeur) routeurMail).createNewConnection(
				serviceWebModelOut,
				((Routeur) routeurMail).getMaLstRule().get(0));
		//ROUTER MAIL 2
		((Routeur) routeurMail).setPropertyValue(
				"2content / property", "1"); //property
		((Routeur) routeurMail).setPropertyValue(
				"2property name", "2");
		((Routeur) routeurMail).setPropertyValue(
				"2property value", "@hotmail.fr");
		((Routeur) routeurMail).setPropertyValue(
				"2comparaison type", "1"); //egal
		((Routeur) routeurMail).createNewConnection(
				mailModelOut,  ((Routeur) routeurMail).getMaLstRule().get(1));
		//ROUTER FTP 1
		routeurFtp.setName("FTP router");
		((Routeur) routeurFtp).setPropertyValue("number of comparaison", "2");
		//
		((Routeur) routeurFtp).setPropertyValue(
				"1content / property", "0"); //content
		((Routeur) routeurFtp).setPropertyValue(
				"1XPATH rule", "/top[value='0']");
		((Routeur) routeurFtp).createNewConnection(
				serviceWebModelOut,
				((Routeur) routeurFtp).getMaLstRule().get(0));
		//ROUTER FTP 2
		((Routeur) routeurFtp).setPropertyValue(
				"2content / property", "0"); //content
		((Routeur) routeurFtp).setPropertyValue(
				"2XPATH rule", "/top[value='1']");
		((Routeur) routeurFtp).createNewConnection(
				mailModelOut,  ((Routeur) routeurFtp).getMaLstRule().get(1));
		//ROUTER RSS 1
		routeurRss.setName("RSS router");
		((Routeur) routeurRss).setPropertyValue("number of comparaison", "2");
		//
		((Routeur) routeurRss).setPropertyValue(
				"1content / property", "0"); //content
		((Routeur) routeurRss).setPropertyValue(
				"1XPATH rule", "/top[value='0']");
		((Routeur) routeurRss).createNewConnection(
				serviceWebModelOut,
				((Routeur) routeurRss).getMaLstRule().get(0));
		//ROUTER RSS 2
		((Routeur) routeurRss).setPropertyValue(
				"2content / property", "0"); //content
		((Routeur) routeurRss).setPropertyValue(
				"2XPATH rule", "/top[value='1']");
		((Routeur) routeurRss).createNewConnection(
				mailModelOut,
				((Routeur) routeurRss).getMaLstRule().get(1));
		//ADDING
		try {
			//INPUT
			diagram.addNode(ftpModel);
			diagram.addNode(rssModel);
			diagram.addNode(mailModelIn);
			//MIDDLE
			diagram.addNode(routeurFtp);
			diagram.addNode(routeurMail);
			diagram.addNode(routeurRss);
			//OUTPUT
			diagram.addNode(serviceWebModelOut);
			diagram.addNode(mailModelOut);
		} catch (Exception e) {
			e.printStackTrace();
		}
		//ENDING
		return diagram;
	}

	/**
	 * this function create an diagram with the RSS Binding sample
	 * in the new file.
	 * @return Diagram
	 */
	private Diagram createRSSBindingSampleDiagram() {
		//
		Diagram diagram = new Diagram();
		//INPUT
		GeneriqueCimeroModel rssModelIn = new RSS();
		//OUTPUT
		GeneriqueCimeroModel screenOutputOut = new ScreenOutput();
		//LOCATION INPUT
		rssModelIn.setLocation(new Point(100, 250));
		//LOCATION OUTPUT
		screenOutputOut.setLocation(new Point(750, 150));
		//CONNECTIONS
		new Connection(rssModelIn, screenOutputOut);
		//CONFIGURATION
		//RSS IN
		rssModelIn.setName("My input RSS");
		rssModelIn.setPropertyValue("polledfrequence", "5000");
		rssModelIn.setPropertyValue("urls"
				, "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/"
				+ "front_page/rss.xml"
				+ ";http://rss.cnn.com/rss/cnn_topstories.rss");
		//SCREENOUTPUT OUT
		//nothing
		//ADDING
		try {
			//INPUT
			diagram.addNode(rssModelIn);
			//OUTPUT
			diagram.addNode(screenOutputOut);
		} catch (Exception e) {
			e.printStackTrace();
		}
		//ENDING
		return diagram;
	}

	/**
	 * this function create an diagram with the basic sample
	 * in the new file.
	 * @return Diagram
	 */
	private Diagram createBasicSampleDiagram() {
		//
		Diagram diagram = new Diagram();
		//INPUT 1
		GeneriqueCimeroModel timerIn1 = new Timer();
		//OUTPUT 1
		GeneriqueCimeroModel jmsOut1 = new JMS();
		//INPUT 2
		GeneriqueCimeroModel jmsIn2 = new JMS();
		//OUTPUT 2
		GeneriqueCimeroModel jmsOut2 = new JMS();
		//INPUT 3
		GeneriqueCimeroModel jmsIn3 = new JMS();
		//OUTPUT 3
		GeneriqueCimeroModel screenOutput = new ScreenOutput();
		//LOCATION INPUT
		timerIn1.setLocation(new Point(100, 150));
		//LOCATION OUTPUT
		jmsOut1.setLocation(new Point(650, 150));
		//LOCATION INPUT
		jmsIn2.setLocation(new Point(100, 350));
		//LOCATION OUTPUT
		jmsOut2.setLocation(new Point(650, 350));
		//LOCATION INPUT
		jmsIn3.setLocation(new Point(100, 550));
		//LOCATION OUTPUT
		screenOutput.setLocation(new Point(650, 550));
		//MEP
		((BindingComponentModel) jmsOut1).setInput(true);
		((BindingComponentModel) jmsOut1).setOutput(false);
		((BindingComponentModel) jmsOut2).setInput(true);
		((BindingComponentModel) jmsOut2).setOutput(false);
		//CONNECTIONS
		new Connection(timerIn1, jmsOut1);
		new Connection(jmsIn2, jmsOut2);
		new Connection(jmsIn3, screenOutput);
		//CONFIGURATION
		//INPUT 1
		timerIn1.setName("My input timer");
		timerIn1.setPropertyValue("message name", "My example job");
		timerIn1.setPropertyValue("message group", "ServiceMix");
		//OUTPUT 1
		jmsOut1.setName("Input Sender");
		jmsOut1.setPropertyValue(
				"provider destination name", "demo.org.servicemix.source");
		jmsOut1.setPropertyValue(
				"jndi connection factory name", "connectionFactory");
		jmsOut1.setPropertyValue("style", 0);
		//INPUT 2
		jmsIn2.setName("Input Receiver");
		jmsIn2.setPropertyValue(
				"provider destination name", "demo.org.servicemix.source");
		jmsIn2.setPropertyValue(
				"jndi connection factory name", "connectionFactory");
		jmsIn2.setPropertyValue("style", 0);
		//OUTPUT 2
		jmsOut2.setName("Output Sender");
		jmsOut2.setPropertyValue(
				"provider destination name", "demo.org.servicemix.result");
		jmsOut2.setPropertyValue(
				"jndi connection factory name", "connectionFactory");
		jmsOut2.setPropertyValue("style", 0);
		//INPUT 3
		jmsIn3.setName("JMS Trace");
		jmsIn3.setPropertyValue(
				"provider destination name", "demo.org.servicemix.result");
		jmsIn3.setPropertyValue(
				"jndi connection factory name", "connectionFactory");
		jmsIn3.setPropertyValue("style", 0);
		//OUTPUT 3
		//nothing
		//SCREENOUTPUT OUT
		//nothing
		//ADDING
		try {
			diagram.addNode(timerIn1);
			diagram.addNode(jmsOut1);
			diagram.addNode(jmsIn2);
			diagram.addNode(jmsOut2);
			diagram.addNode(jmsIn3);
			diagram.addNode(screenOutput);
		} catch (Exception e) {
			e.printStackTrace();
		}
		//ENDING
		return diagram;
	}

	/**
	 * this function create an diagram with the JMS Binding sample
	 * in the new file.
	 * @return Diagram
	 */
	private Diagram createJMSBindingSampleDiagram() {
		//
		Diagram diagram = new Diagram();
		//INPUT
		GeneriqueCimeroModel jmsIn = new JMS();
		//OUTPUT
		GeneriqueCimeroModel jmsOut = new JMS();
		//MEP
		((BindingComponentModel) jmsOut).setInput(true);
		((BindingComponentModel) jmsOut).setOutput(false);
		//LOCATION INPUT
		jmsIn.setLocation(new Point(100, 150));
		//LOCATION OUTPUT
		jmsOut.setLocation(new Point(650, 150));
		//CONNECTIONS
		new Connection(jmsIn, jmsOut);
		//CONFIGURATION
		//JMS IN
		jmsIn.setName("MyConsumer");
		jmsIn.setPropertyValue(
				"provider destination name", "demo.org.servicemix.source");
		jmsIn.setPropertyValue(
				"jndi connection factory name", "connectionFactory");
		jmsIn.setPropertyValue("style", 1);
		//JMS OUT
		jmsOut.setName("MyProvider");
		jmsOut.setPropertyValue(
				"provider destination name", "demo.org.servicemix.result");
		jmsOut.setPropertyValue(
				"jndi connection factory name", "connectionFactory");
		jmsOut.setPropertyValue("style", 1);
		//SCREENOUTPUT OUT
		//nothing
		//ADDING
		try {
			//INPUT
			diagram.addNode(jmsIn);
			//OUTPUT
			diagram.addNode(jmsOut);
		} catch (Exception e) {
			e.printStackTrace();
		}
		//ENDING
		return diagram;
	}

	/**
	 * this function create an diagram with the Quartz Binding sample
	 * in the new file.
	 * @return Diagram
	 */
	private Diagram createQuartzBindingSampleDiagram() {
		//
		Diagram diagram = new Diagram();
		//INPUT
		GeneriqueCimeroModel timer = new Timer();
		//OUTPUT
		GeneriqueCimeroModel screenOutput = new ScreenOutput();
		//LOCATION INPUT
		timer.setLocation(new Point(100, 150));
		//LOCATION OUTPUT
		screenOutput.setLocation(new Point(650, 150));
		//CONNECTIONS
		new Connection(timer, screenOutput);
		//CONFIGURATION
		//TIMER IN
		timer.setName("My input timer");
		timer.setPropertyValue("message name", "My example job");
		timer.setPropertyValue("message group", "ServiceMix");
		//SCREENOUTPUT OUT
		//nothing
		//ADDING
		try {
			//INPUT
			diagram.addNode(timer);
			//OUTPUT
			diagram.addNode(screenOutput);
		} catch (Exception e) {
			e.printStackTrace();
		}
		//ENDING
		return diagram;
	}
}
