package it.eng.cimero.plugin.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.resource.JFaceColors;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ISelectionValidator;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.eclipse.ui.internal.ide.IDEWorkbenchMessages;
import org.eclipse.ui.internal.ide.IIDEHelpContextIds;
import org.eclipse.ui.internal.ide.misc.ContainerSelectionGroup;

/**
 * A standard selection dialog which solicits a container resource from the user.
 * The <code>getResult</code> method returns the selected container resource.
 * <p>
 * This class may be instantiated; it is not intended to be subclassed.
 * </p>
 * <p>
 * Example:
 * <pre>
 * FileSelectionDialog dialog =
 *    new FileSelectionDialog(getShell(), initialSelection, allowNewContainerName(), msg);
 *	dialog.open();
 *	Object[] result = dialog.getResult();
 * </pre> 	
 * </p>
 */
public class FileSelectionDialog extends SelectionDialog {
    // the widget group;
    FileSelectionGroup group;

    // the root resource to populate the viewer with
    private IContainer initialSelection;

    private IFile selectedFile = null;

    // the validation message
    Label statusMessage;

    //for validating the selection
    ISelectionValidator validator;
    
    private String patterns = null;
    
    /**
     * Creates a resource container selection dialog rooted at the given resource.
     * All selections are considered valid. 
     *
     * @param parentShell the parent shell
     * @param initialRoot the initial selection in the tree
     * @param allowNewContainerName <code>true</code> to enable the user to type in
     *  a new container name, and <code>false</code> to restrict the user to just
     *  selecting from existing ones
     * @param message the message to be displayed at the top of this dialog, or
     *    <code>null</code> to display a default message
     */
    public FileSelectionDialog(Shell parentShell, IContainer initialRoot,
            String message, String patterns) {
        super(parentShell);
        setTitle(IDEWorkbenchMessages.FileSelectionDialog_message);
        this.initialSelection = initialRoot;
       
        if (message != null) {
			setMessage(message);
		} else {
			setMessage(IDEWorkbenchMessages.FileSelectionDialog_message);
		}
        this.patterns = patterns;
        setShellStyle(getShellStyle() | SWT.RESIZE);
    }

    /* (non-Javadoc)
     * Method declared in Window.
     */
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
       // PlatformUI.getWorkbench().getHelpSystem()
       //         .setHelp(shell, IIDEHelpContextIds.CONTAINER_SELECTION_DIALOG);
    }

    /* (non-Javadoc)
     * Method declared on Dialog.
     */
    protected Control createDialogArea(Composite parent) {
        // create composite 
        Composite area = (Composite) super.createDialogArea(parent);

       
        Listener listener = new Listener() {
            public void handleEvent(Event event) {
            	IResource resource = group.getSelectedResource();
                if (resource instanceof IFile){
                	selectedFile = (IFile)resource; 
                	getOkButton().setEnabled(true);
                }else{
                	getOkButton().setEnabled(false);
                }
            }
        };
    
        
        // container selection group
        group = new FileSelectionGroup(area, this.initialSelection,listener,
                getMessage(), patterns);
        
        
        statusMessage = new Label(parent, SWT.NONE);
        statusMessage.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        statusMessage.setFont(parent.getFont());

        return dialogArea;
    }

    /**
     * The <code>FileSelectionDialog</code> implementation of this 
     * <code>Dialog</code> method builds a list of the selected resource containers
     * for later retrieval by the client and closes this dialog.
     */
    protected void okPressed() {
    	
        List fileList = new ArrayList();
        
        if (selectedFile != null) {
        	fileList.add(selectedFile);
		}
        
        setResult(fileList);
        super.okPressed();
    }

    /**
     * Sets the validator to use.  
     * 
     * @param validator A selection validator
     */
    public void setValidator(ISelectionValidator validator) {
        this.validator = validator;
    }

    
}
